#############################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0.1.2-1307  USA
#############################################################################

import sys
import traceback

from sys import stdout, stderr
from cStringIO import StringIO

from clive.extract import *
from clive.util import *


__all__ = [
  'Clive',
  'main'
]


class Clive:
  """Command line interface for clive"""
  def __init__(self):
    """Set custom exception hook and start extraction"""
    sys.excepthook = self._exchook

    e = Extract(self._onsay, self._onprogress, self._onprompt)
    e.get()

  # Non-public

  def _onsay(self, msg):
    """[callback] Called to print messages to stdout"""
    msg = msg.rstrip('\r\n')
    msg += '\n'
    if msg.lower().find('error:') == -1:
      stdout.write(msg)
    else:
      stderr.write(msg)

  def _onprogress(self, blocks, blocksize, total):
    """[callback] Called to print progress (stdout)"""
    read = blocks*blocksize
    if total == -1:
      # Often the video page retrievals do not return any 'total' length
      # Print only the amount of bytes we've read so far
      fmt = 'Read : %s' % (self._bytesmb(read))
      progress = -1
    else:
      progress = self._percent(read,total)
      if progress > 100: progress = 100 # Tweak may be needed
      fmt = 'Read : %s of %s (%03d%%)' % (
        self._bytesmb(read),
        self._bytesmb(total),
        progress
      )
    bspaces = '\b' * len(fmt)
    stdout.write(bspaces + fmt)
    stdout.flush()

  def _onprompt(self, msg):
    """[callback] Called to prompt the user for input"""
    return raw_input(msg)

  def _exchook(self, type, value, tb):
    """Custom exception handler, omits ctrl+c, ctrl+d"""
    if type == IOError:
      msg = 'Error: %s' % (value[1][1])
      self._onsay(msg)
    elif type != EOFError and type != KeyboardInterrupt:
      msg = 'Error: %s - %s' % (type,value)
      self._onsay(msg)
      a = self._onprompt('Input: Show traceback? (Y/n):')
      if a.lower() != 'n':
        trace = StringIO()
        traceback.print_exception(type, value, tb, None, trace)
        print trace.getvalue()
    else:
      print

  def _bytesmb(self,bytes):
    """Returns bytes as MB """
    return "%.3fMB" % (float(bytes)/(1024*1024))

  def _byteshuman(self,bytes):
    """Returns bytes as 'human readable size'"""
    a = ['bytes','KB','MB','GB','TB']
    i = int(math.floor(math.log(bytes,1024)))
    b = round(float(bytes/math.pow(1024,i)),2)
    return '%.2f%s' % (b,a[i])

  def _percent(self,i,j):
    """Returns percent"""
    return int(float(i)/float(j)*100)


def main():
  """Program entry point"""
  Clive()
